# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2010 Citrix Systems, Inc.
# Copyright 2010 OpenStack LLC.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os

from nova import log as logging

from nova.virt.xenapi.vm_utils import VMHelper

XenAPI = None
LOG = logging.getLogger("nova.virt.xenapi.xs_agent_driver")


def inject_network_info(vmops, instance, network_info, vm_ref=None):
    """
    Generate the network info and make calls to place it into the
    xenstore and the xenstore param list.
    vm_ref can be passed in because it will sometimes be different than
    what VMHelper.lookup(session, instance.name) will find (ex: rescue)
    """
    logging.debug("Performing injection using default agent xenstore paths")
    if vm_ref:
        # this function raises if vm_ref is not a vm_opaque_ref
        vmops._session.get_xenapi().VM.get_record(vm_ref)
    else:
        vm_ref = VMHelper.lookup(vmops._session, instance.name)

    for (network, info) in network_info:
        logging.debug(_("Injecting IP configuration for MAC:%s"), info['mac'])
        location = 'vm-data/networking/%s' % info['mac'].replace(':', '')
        vmops.write_to_param_xenstore(vm_ref, {location: info})
        try:
            # TODO(tr3buchet): fix function call after refactor
            # vmops.write_to_xenstore(vm_ref, location, info)
            vmops._make_plugin_call('xenstore.py', 'write_record', instance,
                                   location, {'value': json.dumps(info)},
                                   vm_ref)
        except KeyError:
            # catch KeyError for domid if instance isn't running
            pass
