# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2010 Citrix Systems, Inc.
# Copyright 2010 OpenStack LLC.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from nova import log as logging

from nova.virt.xenapi.vm_utils import VMHelper

XenAPI = None
LOG = logging.getLogger("nova.virt.xenapi.xs_agent_driver")


def inject_network_info(vmops, instance, network_info, vm_ref=None):
    """
    Generate the network info and make calls to place it into the
    vm-data area of the xenstore.
    """
    LOG.debug("Performing injection using xenserver agent xenstore paths")

    xenstore_types = {
        #'BroadcastAddress': 'multi_sz',
        'DefaultGateway': 'multi_sz',
        'EnableDhcp': 'dword',
        'IPAddress': 'multi_sz',
        'NameServer': 'string',
        'SubnetMask': 'multi_sz'}

    if vm_ref:
        # this function raises if vm_ref is not a vm_opaque_ref
        vmops._session.get_xenapi().VM.get_record(vm_ref)
    else:
        vm_ref = VMHelper.lookup(self._session, instance.name)

    for (network, info) in network_info:
        LOG.debug(_("Injecting IP configuration for MAC:%s"), info['mac'])

        xenstore_prefix = ('vm-data/vif/' +
            info['mac'].replace(':', '_').upper() + '/tcpip/')
        xenstore_data = {
            # NB: Setting broadcast address is not supported by
            # Windows or the Windows guest agent, and will be ignored
            # on that platform
            #'BroadcastAddress': info['broadcast'],
            'EnableDhcp': '0',
            'IPAddress': info['ips'][0]['ip'],
            'SubnetMask': info['ips'][0]['netmask'],
            'DefaultGateway': info['gateway'],
            'NameServer': info['dns'][0]}

        for xenstore_key, xenstore_value in xenstore_data.iteritems():
            # NB: The xenstore_key part of the instance_key isn't used but
            # must be unique.  We set it to xenstore_key as a convenient
            #  unique name...The xenstore_key value takes effect in the
            # /name element.
            instance_key = xenstore_prefix + xenstore_key
            key_type = xenstore_types[xenstore_key]
            vmops.write_to_param_xenstore(vm_ref,
                {instance_key + '/name': xenstore_key},
                False)
            vmops.write_to_param_xenstore(vm_ref,
                {instance_key + '/type': key_type},
                False)
            if key_type == 'multi_sz':
                vmops.write_to_param_xenstore(vm_ref,
                    {instance_key + '/data/0': xenstore_value},
                    False)
            else:
                vmops.write_to_param_xenstore(vm_ref,
                    {instance_key + '/data': xenstore_value},
                    False)
